    .LI     OFF
;---------------------------------------------------------------------
;
; Ikea Striberg delay
;
; The Ikea Striberg wardrobe LED strip automatically switches on when
; the wardrobe is opened. Its IR proximity sensor is quite sensitive
; and is supposed to look over the person standing in front of the
; wardrobe.
; However if you hang the strip too low a person standing in front of
; the wardrobe makes the light go off again.
; The purpose of this processor is to delay the off switching by a
; set number of seconds, making the sensor less nervous.
;
; Author: San Bergmans
; Date:   Januari 2018
;
; www.sbprojects.net
;
;---------------------------------------------------------------------

    .IN     p12f629
    .TF     ikea.hex,int

;---------------------------------------------------------------------
; GPIO assignment
;
; GPIO3 = Input from N2 pin 3 (GP4) High is lights on
; GPIO4 = Output to base of Q6
;---------------------------------------------------------------------

IKEA_OUT    =   GPIO4       Output to gate of Q6 (through R25)
IKEA_IN     =   GPIO5       Input from N01 pin 3 (High is on)

;---------------------------------------------------------------------
; RAM assignment
;---------------------------------------------------------------------

        .SM     RAM
        .OR     $20

TIMER_10MS  .BS 2           Counter to delay 10ms
TIMER_1SEC  .BS 1           Counter to delay 1s
TIMER_LIGHT .BS 1           Counter to delay x number of seconds

;---------------------------------------------------------------------
;  Constants
;---------------------------------------------------------------------

DELAY_TIME  =   30          Light remains on this number of seconds

;---------------------------------------------------------------------
;  Initialize
;---------------------------------------------------------------------

        .SM     CODE
        .OR     0

RESET
    MOVLW   #%1000.0000     Select pull-ups, the rest is don't care
    MOVWF   OPTION_REG

    MOVLW   #%0000.0000     Disable all interrupts
    MOVWF   INTCON
    MOVWF   PIE1

    BSF     STATUS,RP0      Calibrate the oscillator
    CALL    $03FF
    MOVWF   OSCCAL

    MOVLW   #%0000.0111     Disable comparator mode
    MOVWF   CMCON
    BSF     STATUS,RP0      Initialize GPIO
    MOVLW   #%0010.0000     All pins are outputs, except GPIO5
    MOVWF   TRISIO
    BCF     STATUS,RP0

    MOVLW   #%0000.0000     Set initial state of the output
    MOVWF   GPIO

;---------------------------------------------------------------------
;  Main program loop
;
;  If light goes off, keep it on for set amount of time
;---------------------------------------------------------------------

MAIN
    BTFSS   GPIO,IKEA_IN    Is the input low?
    GOTO    MAIN            Yes! Nothing to do

    CALL    DELAY10MS       Debounce the input
    BTFSS   GPIO,IKEA_IN    Is the input low again?
    GOTO    MAIN            Yes! It was only a spike. We're done

    BSF     GPIO,IKEA_OUT   Output high, turn light on

.ON_AGAIN
    MOVLW   #DELAY_TIME     Set delay time in seconds
    MOVWF   TIMER_LIGHT

.ON_LOOP
    MOVLW   #100            Set seconds timer
    MOVWF   TIMER_1SEC
.SECONDS
    CALL    DELAY10MS       Delay 10 ms
    BTFSC   GPIO,IKEA_IN    Is input high, still or again?
    GOTO    .ON_AGAIN       Yes! Restart the timer
    DECFSZ  TIMER_1SEC
    GOTO    .SECONDS        Repeat loop for 1 second

    DECFSZ  TIMER_LIGHT
    GOTO    .ON_LOOP        Repeat until dealy time is over!

    BCF     GPIO,IKEA_OUT   Switch light off, it's been on long enough
    GOTO    MAIN

;---------------------------------------------------------------------
;  DELAY10MS
;  Delays 10 milli second
;  No input or output parameters required
;---------------------------------------------------------------------

DELAY10MS
        MOVLW   16              Initialize outer loop
        MOVWF   TIMER_10MS
.LOOP1
        MOVLW   207             Initialize inner loop
        MOVWF   TIMER_10MS+1
.LOOP2
        DECFSZ  TIMER_10MS+1
        GOTO    .LOOP2          Repeat inner loop

        DECFSZ  TIMER_10MS
        GOTO    .LOOP1          Repeat outer loop

        RETURN


;---------------------------------------------------------------------
;  Option bits
;---------------------------------------------------------------------

    .OR     $2007
;           b13-b12 Don't care about band gap voltage
;           b11-b9  Unimplemented
;           b8      Data code protection disabled
;           b7      Program memory code protection disabled
;           b6      Brown-out detection disabled
;           b5      No RESET input on GP3
;           b4      Power-up timer enabled
;           b3      Watchdog disabled
;           b2-b0   INTOSC - normal GPIO functions
    .CW     %00.0001.1000.0100
